import 'package:flutter/scheduler.dart';
import 'package:get/get.dart';
import 'package:uuid/uuid.dart';

class StatusController extends GetxController {
  static Duration delayForCompletion = Duration(milliseconds: 500);

  RxList<DartError> _exceptions = <DartError>[].obs;

  Set<Uuid> processes = <Uuid>{};
  RxBool isProcessing = false.obs;

  void add(Uuid uuid) {
    processes.add(uuid);
    _rebuildIfNeeded();
  }

  void complete(Uuid uuid) {
    processes.remove(uuid);
    _rebuildIfNeeded(delay: delayForCompletion);
  }

  void _rebuildIfNeeded({Duration delay = const Duration()}) {
    SchedulerBinding.instance?.addPostFrameCallback((timeStamp) {
      if (isProcessing.value == processes.isNotEmpty) return;
      Future.delayed(delay).then((value) => isProcessing.toggle());
    });
  }

  DartError? currentError() => _exceptions.isEmpty ? null : _exceptions.first;

  void catchError(dynamic error) {
    _exceptions.add(DartError(error));
  }

  void consumeError(DartError error) {
    _exceptions.removeWhere((element) => element.uuid == error.uuid);
  }
}

class DartError {
  dynamic error;
  String uuid = Uuid().v4();

  DartError(this.error);

  @override
  bool operator ==(Object other) =>
      (other is DartError) ? uuid == other.uuid : false;

  @override
  String toString() => error.toString();
}
